-- get engine environment
local A, O = unpack(select(2, ...))
-- set engine as new global environment
setfenv(1, _G.AlertMe)
-- create table for snapshots
A.Snapshots = {}

function A:InitLCD()
	dprint(3, "A:InitLCD")
	A.Libs.LCD:Register("AlertMe")
	A.Libs.LCD.enableEnemyBuffTracking = true
	UnitAura = A.Libs.LCD.UnitAuraWithBuffs
	--A.Libs.LCD.RegisterCallback("AlertMe", "UNIT_BUFF", function(event, unit) end)
end

function A:GetUnitAura(ti, eventInfo)
	dprint(3, "A:GetUnitAura", ti.relSpellName, ti.dstName)
	local unit = (ti.dstIsTarget == true) and "target" or ti.dstName
	local filter = (ti.auraType == "BUFF") and "HELPFUL" or "HARMFUL"
	return A:MatchUnitAura(ti, eventInfo, unit, filter)
	-- if not name and not ti.delayed then -- only do the first timer
	-- 	C_Timer.After(0.2, function()
	-- 		A:ProcessTriggerInfo(ti, eventInfo)
	-- 	end)
end

function A:MatchUnitAura(ti, eventInfo, unit, filter)
	dprint(3, "A:MatchUnitAura", ti.relSpellName, "unit", unit, "filter", filter)
	for i = 1, 100 do
		local name, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellId  = UnitAura(unit, i, filter)
		if not name then
			break
		elseif ti.relSpellName == name then
			local remaining = (expirationTime > 0) and expirationTime - GetTime() or 0
			return name, icon, count, debuffType, duration, expirationTime, source, isStealable, nameplateShowPersonal, spellId, remaining
		end
	end
end

function A:FakeEvent(ti, eventInfo)
	dprint(3, "A:FakeEvent", ti.relSpellName, eventInfo.short)
	-- create fake args
	local _ti = tcopy(ti)
	_ti.event = "SPELL_AURA_APPLIED"
	local _eventInfo = A.Events["SPELL_AURA_APPLIED"]
	-- get alerts for fake args
	local check, alerts = A:DoChecks(_ti, _eventInfo)
	if not check then return end
	-- check for snapshots
	local exists
	exists, _ti, _eventInfo = A:CheckSnapShot(ti, eventInfo)
	if exists then
		-- do whatever is defined in actions
		dprint(3, "A:FakeEvent", "do actions", _ti.relSpellName, _eventInfo.short)
		A:DoActions(_ti, _eventInfo, alerts, true)
	else
		-- if no snapshot was found, add one for cast success event
		A:AddSnapShot(ti, eventInfo)
	end
end

function A:CheckSnapShot(ti, eventInfo)
	dprint(3, "A:CheckSnapShot", ti.relSpellName, eventInfo.short)
	A:CleanSnapshots()
	if eventInfo.short == "gain" then
		if A.Snapshots[ti.dstGUID] and A.Snapshots[ti.dstGUID][ti.relSpellName] and A.Snapshots[ti.dstGUID][ti.relSpellName]["success"] then
			local snapShot = A.Snapshots[ti.dstGUID][ti.relSpellName]["success"]
			local timeDiff = GetTime() - snapShot.ts
			if timeDiff >= 0 and timeDiff < 2 then
				--dprint(3, "A:CheckSnapShot", "aura gain: found match", timeDiff, ti.relSpellName, eventInfo.short)
				return true, snapShot.ti, snapShot.eventInfo
			end
		end
		--dprint(3, "A:CheckSnapShot", "aura gain: no match", timeDiff, ti.relSpellName)
		return false
	elseif eventInfo.short == "success" then
		if A.Snapshots[ti.dstGUID] and A.Snapshots[ti.dstGUID][ti.relSpellName] and A.Snapshots[ti.dstGUID][ti.relSpellName]["gain"] then
			local snapShot = A.Snapshots[ti.dstGUID][ti.relSpellName]["gain"]
			local timeDiff = GetTime() - snapShot.ts
			if timeDiff >= 0 and timeDiff < 2 then
				dprint(3, "A:CheckSnapShot","cast success: found match", timeDiff, ti.relSpellName)
				return true, snapShot.ti, snapShot.eventInfo
			end
		end
		--dprint(3, "A:CheckSnapShot", "cast success: no match", ti.relSpellName, eventInfo.short)
		return false
	end
end

function A:AddSnapShot(ti, eventInfo)
	dprint(3, "A:AddSnapShot", ti.relSpellName, eventInfo.short)
	if not A.Snapshots[ti.dstGUID] then A.Snapshots[ti.dstGUID] = {} end
	if not A.Snapshots[ti.dstGUID][ti.relSpellName] then A.Snapshots[ti.dstGUID][ti.relSpellName] = {} end
	A.Snapshots[ti.dstGUID][ti.relSpellName][eventInfo.short] = {
		ts = GetTime(),
		ti = tcopy(ti),
		eventInfo = tcopy(eventInfo)
	}
end

function A:CleanSnapshots()
	dprint(3, "A:CleanSnapshots")
	local now = GetTime()
	for d, dstGUID in pairs(A.Snapshots) do
		for s, relSpellName in pairs(dstGUID) do
			for e, short in pairs(relSpellName) do
				if now - short.ts > 10 then
					A.Snapshots[d][s][e] = nil
				end
			end
		end
	end
	for d, dstGUID in pairs(A.Snapshots) do
		for s, relSpellName in pairs(dstGUID) do
			if not next(relSpellName) then
				A.Snapshots[d][s] = nil
			end
		end
	end
	for d, dstGUID in pairs(A.Snapshots) do
		if not next(dstGUID) then
			A.Snapshots[d] = nil
		end
	end
end
